class Cereal {
  private String name;
  protected int sugarPerServing; // in grams
  protected String[] ingredients;
  protected double servingSize; // in cups

  String getName() {
    return name;
  }

  Cereal(String name) {
    this.name = name;
  }
}

class FrootLoops extends Cereal {
  FrootLoops() {
    super("Froot Loops");
    sugarPerServing = 12;
    servingSize = 4.0 / 3.0;
    ingredients = new String[]{
      "Corn flour blend",
      "Sugar",
      "Wheat flour",
      "Whole grain oat flour",
      "Modified food starch"
    };
  }
}

class HoneyNutCheerios extends Cereal {
  HoneyNutCheerios() {
    super("Honey Nut Cheerios");
    sugarPerServing = 12;
    servingSize = 1.0;
    ingredients = new String[]{
      "Whole Grain Oats",
      "Sugar",
      "Corn Starch",
      "Honey",
      "Brown Sugar Syrup",
      "Salt"
    };
  }
}

class Box<T extends Cereal> {
  private T cereal;
  Box(T cereal) {
    this.cereal = cereal;
  }

  Box() {
    cereal = null;
  }

  void setCereal(T cereal) {
    this.cereal = cereal;
  }

  public String getLabel() {
    String label = cereal.getName() + ":\n";
    label += "\tServing size: " + cereal.servingSize + " cups\n";
    label += "\tSugar per serving: " + cereal.sugarPerServing + " grams\n";
    label += "\tIngredients: ";
    for (int i = 0; i < cereal.ingredients.length; i++) {
      label += cereal.ingredients[i];
      if (i + 1 < cereal.ingredients.length) label += ", ";
    }
    label += "\n";
    return label;
  }

  public String toString() {
    return getLabel();
  }
}

public class GenericsExamples {
  public static void main(String[] args) {

    Box<FrootLoops> frootLoopsBox = new Box<FrootLoops>(new FrootLoops());
    System.out.println(frootLoopsBox);

    // Create a box, but don't pass in the cereal to save just yet.
    // Note that we can omit the type in between < >  when *constructing*
    // the parametrized box (but we still need to include the type when
    // *declaring* the cheeriosBox variable).
    Box<HoneyNutCheerios> cheeriosBox = new Box<>();
    HoneyNutCheerios cheerios = new HoneyNutCheerios();
    cheeriosBox.setCereal(cheerios); // now pass in the cereal
    System.out.println(cheeriosBox);
  }
}